import { splitTokens } from '@shikijs/core';
import React from 'react';
import { View, ActivityIndicator, Text, ScrollView } from 'react-native';
import php from '../node_modules/shiki/dist/langs/php.js';
import tsx from '../node_modules/shiki/dist/langs/tsx.js';
import typescript from '../node_modules/shiki/dist/langs/typescript.js';
import { getHighlighter } from './SyntaxContext.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const LANGS = {
  php,
  tsx,
  typescript
  // Add more languages as needed
};
function log(message, ...args) {
  {
    console.log(`[SyntaxHighlighter] ${message}`, ...args);
  }
}
function extractPatternsFromGrammar(grammar) {
  const repository = (grammar == null ? void 0 : grammar.repository) || {};
  const breakpoints = /* @__PURE__ */ new Set();
  function processPatterns(patterns) {
    patterns.forEach((pattern) => {
      if (pattern.include) {
        const key = pattern.include.replace("#", "");
        if (repository[key]) {
          if (repository[key].patterns)
            processPatterns(repository[key].patterns);
          if (repository[key].begin)
            breakpoints.add(0);
          if (repository[key].match)
            breakpoints.add(repository[key].match.length || 0);
        }
      } else {
        if (pattern.begin) {
          breakpoints.add(0);
          if (pattern.end)
            breakpoints.add(pattern.end.length || 0);
        }
        if (pattern.match)
          breakpoints.add(pattern.match.length || 0);
        if (pattern.patterns)
          processPatterns(pattern.patterns);
      }
    });
  }
  if (grammar.patterns)
    processPatterns(grammar.patterns);
  return breakpoints;
}
function processTokens(tokens, language) {
  const grammar = LANGS[language];
  if (!grammar) {
    log("No grammar found for language:", language);
    return tokens;
  }
  const grammarBreakpoints = extractPatternsFromGrammar(language);
  tokens.forEach((line) => {
    line.forEach((token) => {
      const start = token.offset || 0;
      const end = start + token.content.length;
      grammarBreakpoints.add(start);
      grammarBreakpoints.add(end);
      if (token.content.length > 1) {
        token.content.split(/([(){}[\]<>.,;:?=+\-*/%&|^!~`@#$]+)/).forEach((part, index, parts) => {
          if (part) {
            const partStart = start + parts.slice(0, index).join("").length;
            grammarBreakpoints.add(partStart);
            grammarBreakpoints.add(partStart + part.length);
          }
        });
      }
    });
  });
  return splitTokens(tokens, grammarBreakpoints);
}
const defaultHighlightLines = [];
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines
}) => {
  var _a;
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  const [tokens, setTokens] = React.useState([]);
  React.useEffect(() => {
    const highlight = () => __async(void 0, null, function* () {
      try {
        log("Starting highlighting process");
        log("Input:", { text, language, theme: theme.name });
        setLoading(true);
        const highlighter = yield getHighlighter();
        log("Highlighter instance obtained");
        if (!text || !language || !theme) {
          throw new Error("Missing required props");
        }
        log("Attempting tokenization...");
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme,
          includeExplanation: true
        });
        log("Raw tokenization result:", result);
        if (!(result == null ? void 0 : result.tokens)) {
          throw new Error("Invalid tokenization result");
        }
        const processedTokens = yield processTokens(result.tokens, language);
        log("Processed tokens:", processedTokens);
        setTokens(processedTokens);
        setError(null);
      } catch (err) {
        const errorMessage = err instanceof Error ? err.message : "Failed to highlight code";
        log("Error during highlighting:", err);
        setError(errorMessage);
        setTokens([]);
      } finally {
        setLoading(false);
      }
    });
    highlight();
  }, [text, language, theme]);
  const renderToken = (token, index) => {
    return /* @__PURE__ */ React.createElement(
      Text,
      {
        key: `${token.offset || index}-${index}`,
        style: [
          {
            color: token.color,
            fontSize,
            fontFamily: "monospace",
            fontStyle: token.fontStyle === 2 ? "italic" : "normal",
            fontWeight: token.fontStyle === 1 ? "bold" : "normal"
          },
          textStyle
        ]
      },
      token.content
    );
  };
  const renderLine = (lineTokens, index) => {
    const isHighlighted = highlightLines.includes(index + 1);
    return /* @__PURE__ */ React.createElement(
      View,
      {
        key: index,
        style: [
          {
            flexDirection: "row",
            flexWrap: "wrap",
            backgroundColor: isHighlighted ? "rgba(255,255,255,0.1)" : "transparent"
          }
        ]
      },
      lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
    );
  };
  if (loading) {
    return /* @__PURE__ */ React.createElement(View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(ActivityIndicator, null));
  }
  if (error) {
    return /* @__PURE__ */ React.createElement(View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(Text, { style: { color: "#ff6b6b" } }, "Error:", error));
  }
  return /* @__PURE__ */ React.createElement(ScrollView, __spreadProps(__spreadValues({}, scrollViewProps), { horizontal: true, style: [{ backgroundColor: ((_a = theme.colors) == null ? void 0 : _a.background) || "#1e1e1e" }, scrollViewProps == null ? void 0 : scrollViewProps.style], contentContainerStyle: [{ padding: 10 }, containerStyle] }), /* @__PURE__ */ React.createElement(View, { onStartShouldSetResponder: () => true }, tokens.map((lineTokens, index) => renderLine(lineTokens, index))));
};

export { SyntaxHighlighter };
//# sourceMappingURL=SyntaxHighlighter.js.map
